import os
import sqlite3
import time
import warnings

import pandas as pd
from tqdm.auto import tqdm
from NGramGeneration import NGramGeneration
from TextAnalysis import TextAnalysis

from TextNormalization import TextNormalization
from TopicModeling import TopicModeling

warnings.filterwarnings("ignore")


def main():
    start = time.time()

    print(" --  -- Started: Read Input Data --  -- ")
    input_data = pd.read_csv('data/Mobile_Security_Questions.csv')
    input_data = input_data.loc[input_data['CreationDate'].str.contains('2018-')]

    print(" --  -- Started: Text Normalization --  -- ")
    input_data['normalized_body'] = None
    input_data['normalized_body_tokens'] = None

    text_normalization = TextNormalization(custom_stopwords_file='input/custom_stopwords.csv')
    for index, row in tqdm(input_data.iterrows(), desc="Text Preprocessing rows", total=len(input_data)):
        tokens = text_normalization.normalize_text(row['Title'])
        text = ' '.join(tokens).strip()
        input_data.at[index, 'normalized_body'] = text
        input_data.at[index, 'normalized_body_tokens'] = tokens

    print(" --  -- Started: Generate Ngram --  -- ")
    ngram_generation = NGramGeneration()
    input_data = ngram_generation.generate_bi_trigrams(input_data)

    print(" --  -- Started: Save Updated Dataset --  -- ")
    if not os.path.exists("output"):
        os.makedirs("output")
    conn = sqlite3.connect('output/input_normalized.sqlite')
    temp_dataset = input_data.copy(deep=True)
    temp_dataset.drop('normalized_body_tokens', 1, inplace=True)
    temp_dataset.drop('bigrams_tokens', 1, inplace=True)
    temp_dataset.drop('trigrams_tokens', 1, inplace=True)
    temp_dataset.drop('fourgrams_tokens', 1, inplace=True)
    temp_dataset.drop('fivegrams_tokens', 1, inplace=True)
    temp_dataset.to_sql('input_normalized', conn, if_exists='replace', index=True)

    print(" --  -- Started: Generate Word Cloud & Frequency --  -- ")
    text_analysis = TextAnalysis()
    text_analysis.generate_word_frequency(input_data['normalized_body'])
    text_analysis.generate_word_frequency(input_data['bigrams_text'])
    text_analysis.generate_word_frequency(input_data['trigrams_text'])

    text_analysis.generate_word_cloud(input_data['normalized_body'])
    text_analysis.generate_word_cloud(input_data['bigrams_text'])
    text_analysis.generate_word_cloud(input_data['trigrams_text'])

    print(" --  -- Started: Topic Modeling --  -- ")
    topic_modeling = TopicModeling('unigrams')
    topic_modeling.generate_topics(input_data['normalized_body_tokens'])

    end = time.time()
    print("Process completed in " + str(end - start) + " seconds")


if __name__ == "__main__":
    main()
